/*GPL*START*
 * 
 * Copyright (C) 1998 by Johannes Overmann <overmann@iname.com>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * *GPL*END*/  

#include <math.h>
#include <qpainter.h>

#include "loadgraph.h"



static const QRgb col[Qjo_loadgraph_NUM_COLORS] = {0x008800, 0xff0000, 0x00ff00, 0x0000ff, 0xaa0000, 0x888888, 0x0000aa, 0x550000, 0x005500, 0x000055};
static const QRgb col_r[Qjo_loadgraph_NUM_COLORS] = {0x008800, 0xff0000, 0x00ff00, 0x0000ff, 0xaa0000, 0x888888, 0x0000aa, 0x775533, 0x005500, 0x335577};

LoadGraph::LoadGraph(QWidget *parent, bool grid, bool reverse, int num, const TString& title, Machine *mach):
QWidget(parent), grid(grid), num(num), title(title), isdirty(true)
{
   int i;
   if(mach) m += mach;
   pix = 0;
   pop = 0;
   if(reverse) {
      c_border.setRgb(0xffeeee);
      c_bg.setRgb(0x0);
      c_fg.setRgb(0xccccff);
      c_grid.setRgb(0x444444);
      for(i=0; i < Qjo_loadgraph_NUM_COLORS; i++) {
	 c_graph[i].setRgb(col_r[i]);
      }
   } else {
      c_border.setRgb(0x440000);
      c_bg.setRgb(0xffffff);
      c_fg.setRgb(0x000088);
      c_grid.setRgb(0xcccccc);
      for(i=0; i < Qjo_loadgraph_NUM_COLORS; i++) {
	 c_graph[i].setRgb(col[i]);
      }
   }
}


LoadGraph::~LoadGraph() {
   if(pix) delete pix;
//   if(pop) delete pop;  // crashes
}


void LoadGraph::paintEvent(QPaintEvent *p) {
   if(isdirty) {
      if(pix==0) pix = new QPixmap(rect().size());
      if(rect()!=pix->rect()) pix->resize(size());
      drawgraph();
   }
   bitBlt(this, p->rect().topLeft(), pix, p->rect());
}


void LoadGraph::resizeEvent(QResizeEvent *) {
   isdirty = true;
}


void LoadGraph::popact(int id) {
   int i = pop->indexOf(id);
   
   if(i<m.num()) {
      m[i]->graph = false;
      m.slowRemove(i);
      pop->removeItem(id);
   } else {
      for(int j=0; j < m.num(); j++) 
	m[j]->graph = false;
      m.empty();
   }
   
   if(m.num()) { // some machines left
      isdirty = true;
      repaint();
   } else { // no machines left
      emit is_empty();
   }   
}


void LoadGraph::mousePressEvent(QMouseEvent *e) {
   if(pop==0) {
	pop = new QPopupMenu;
      for(int i=0; i < m.num(); i++) {
	 pop->insertItem("Hide " + m[i]->name);
      }
      if(m.num()>1) {
	 pop->insertSeparator();
	 pop->insertItem("Hide all");
      }
      connect(pop, SIGNAL(activated(int)), SLOT(popact(int)));
   }
   pop->popup(mapToGlobal(e->pos()));
}


double LoadGraph::load(int mach, int back) {
   if(m[mach]->hist.num() > back) 
     return m[mach]->hist[m[mach]->hist.num() - 1 - back];
   else 
     return UNK_LOAD;
}

void LoadGraph::drawgraph() {
  m.fixedSize();
  
  // config
  int key_y = 15;
  int key_lx = 20;
  int key_ld = 5;
   
  int i, j, y, x;
  int w = pix->width() - 2;
  int h = pix->height() - 2;
  double di;
   
  // setup
  pix->fill(c_bg);
  if(m.num()==0) return;
  QPainter p(pix);
  p.setFont(QFont("helvetica", 12, QFont::Bold));
  double max = 1.0, l;
  for(i=0; i < m.num(); i++) {
	 for(j=0; j < num; j++) {
		l = load(i, j);
		if(l>max) max = l;
	 }
  }
  max=floor((max+0.15)*10.0)/10.0;
  double load2pix = double(pix->height())/max;
  double back2pix = double(w)/(num);
  double gnum=num;
  double gback2pix = back2pix;
  while(gback2pix < 5.0) {
	 gnum/=10.0;
	 gback2pix = double(w)/(gnum);
  }
  
  // draw grid
  if(grid) {
	 for(di=1, i=1; di < gnum; di++, i++) {
		p.setPen(QPen(c_grid, 0, DotLine));   
		if((i%5)==0) p.setPen(QPen(c_grid, 0, DashLine));
		if((i%10)==0) p.setPen(QPen(c_grid, 0, SolidLine));
		p.drawLine(1 + w - int(di*gback2pix) , 1, 1 + w - int(di*gback2pix), 1 + h);
	 }
	 for(di=0.1, i=1; di < max; di+=0.1, i++) {
		if((i%5)&&(load2pix<50.0)) continue;
		if((i%10)&&(load2pix<30.0)) continue;
		p.setPen(QPen(c_grid, 0, DotLine));   
		if((i%5)==0) p.setPen(QPen(c_grid, 0, DashLine));
		if((i%10)==0) p.setPen(QPen(c_grid, 0, SolidLine));
		p.drawLine(1, 1 + h - int(di*load2pix), w + 1, 1 + h - int(di*load2pix));
	 }
  } else { // ticks
	 p.setPen(c_grid);   
	 int len;
	 for(di=1, i=1; di < gnum; di++, i++) {
		len = 1;
		if((i%5)==0) len = 3;
		if((i%10)==0) len = 5;
		p.drawLine(1 + w - int(di*gback2pix) , 1, 1 + w - int(di*gback2pix), 1 + len);
		p.drawLine(1 + w - int(di*gback2pix) , 1+h-len-1, 1 + w - int(di*gback2pix), 1 + h);
	 }
	 for(di=0.1, i=1; di < max; di+=0.1, i++) {
		if((i%5)&&(load2pix<50.0)) continue;
		if((i%10)&&(load2pix<30.0)) continue;
		len = 1;
		if((i%5)==0) len = 3;
		if((i%10)==0) len = 5;
		p.drawLine(1, 1 + h - int(di*load2pix), 1 + len, 1 + h - int(di*load2pix));
		p.drawLine(1 + w-len-1, 1 + h - int(di*load2pix), 1 + w, 1 + h - int(di*load2pix));
	 }
  }
  
  // draw graph
  for(i=0; i < m.num(); i++) {
	 p.setPen(c_graph[i%Qjo_loadgraph_NUM_COLORS]);
	 bool lastvalid=false;
	 int lastx=0;
      int lasty=0;
      for(j=0; j < num; j++) {
		  if(load(i, j)!=UNK_LOAD) {
			 if(lastvalid) {
				p.drawLine(1 + w - int(j*back2pix), 1 + h - int(load(i,j)*load2pix), 1 + lastx, 1 + lasty);
			 }
			 lastx = w - int(j*back2pix);
			 lasty = h - int(load(i,j)*load2pix);
			 lastvalid = true;
		  } else lastvalid = false;
      }
  }
  
  // draw border
  p.setPen(c_border);
  p.drawRect(pix->rect());
   
  // draw key
  w+=2;
  h+=2;
  if(title) {
	 p.setPen(c_fg);      
	 p.drawText(0, 0, w, key_y, AlignCenter|AlignVCenter|SingleLine, title);
  }
  if(1) {
	 for(i=0, y=0, x=0; i < m.num(); i++, y+=key_y) {
		if((y+key_y > h)&&(y>0)) {
		  y=0;
		  x+=100;
		}
		p.setPen(c_fg);
		p.drawText(x+key_lx+2*key_ld, y, w-key_lx-2*key_ld, key_y, AlignLeft|AlignVCenter|SingleLine, m[i]->name);
		p.setPen(c_graph[i%Qjo_loadgraph_NUM_COLORS]);
		p.drawLine(x+key_ld, y+key_y/2, x+key_lx+key_ld, y+key_y/2);
	 }
  }
  isdirty = false;
}


